#ifndef DCSINTERFACE_H
#define DCSINTERFACE_H

#include "ConfigurationInterface.h"
#include "SctData/DcsData.h"
#include <string>
#include <map>
#include <boost/shared_ptr.hpp>

namespace SctAnalysis{
  /** Gets dcs data out of IS. 
      Default is to keep its own configuration interface, 
      but one can also be shared.
  */
  class DcsInterface{
  public:
    /** shares configuration interface */
    DcsInterface(boost::shared_ptr<ConfigurationInterface> config);
    /** destructor */
    ~DcsInterface();
    /// get some DcsData for a particular module!
    boost::shared_ptr<SctData::DcsData> getData(const std::string& modulename) const;
    /// return ConfigurationInterface
    const ConfigurationInterface& getConfigurationInterface() const;
    /// output some basic usage information:
    void printStatus(std::ostringstream&) const;
  private:
    boost::shared_ptr<ConfigurationInterface> m_config; ///< keeps itself up-to-date
    mutable unsigned m_call_count; ///< counts calls asked of it
    mutable unsigned m_val_count; ///< counts values returned
    mutable unsigned m_error_count; ///< counts values returned
    /** not allowed */
    DcsInterface();
  };

} //namespace SctAnalysis


#endif // DCSINTERFACE_H
