#ifndef ANALYSISSERVICE_OPETOOL_H
#define ANALYSISSERVICE_OPETOOL_H

#include <TH1.h>
#include "SctData/RawScanResult.h"
#include "SctData/OpeResult.h"

namespace SctAnalysis {
  /** Analysis of OccupancyPerEvent data. For now, it only accepts the Raw data from NoiseOccupancy but this could be extended in the near future.  */
class OpeTool{
 public:
  OpeTool();
  ~OpeTool();
  /** For each threshold value for each chip, the analysis evaluates the mean and the variance of the distribution. 
   Using propreties of the Binomial distribution, a variance is calculated from the mean. Badness is defined 
  as the ratio of the evaluated variance to the calculated variance. Badness bigger then 2 should be flagged as bad. */
  boost::shared_ptr<SctData::OpeResult> OpeTool::analyzeModule(const SctData::RawScanResult& raw);
  /** */
  double OpeTool::findChi2Binomial(const TH1& data);
  double OpeTool::fractionVariance(const TH1& data); 
 private:
  /** The following have been copied from the latest ROOT libraries since this 
   latest version of ROOT is not necessarily installed -- these are left over from an earlier version.  */
  Double_t OpeTool::BinomialI(Double_t p, Int_t n, Int_t k);
  Double_t OpeTool::BetaIncomplete(Double_t x, Double_t a, Double_t b);
  Double_t OpeTool::Beta(Double_t p, Double_t q);
  Double_t OpeTool::BetaCf(Double_t x, Double_t a, Double_t b);
};

} // end of namespace

#endif
