#include "TestLibraryImpl.h"
#include "Sct/SctNames.h"
#include "Sct/SequenceUtils.h"
#include "Sct/OmniMarshalling.h"
#include "mrs/message.h"

namespace SctCalibrationController {
    
  bool TestLibraryImpl::registerStaticFactoryMethodForTest(const std::string & name,  const FunctionPointer f) {
    std::cout << "Making a note of a TestRequest called [" << name << "] at line " << __LINE__ << " of " << __FILE__ << std::endl;
    bool success; // no need to initialise

    if (f!=0) {
      stringFunctionPtrMap()[name] = f;
      success = true;
    } else {
      std::cerr << "I refuse to make a note of a TestRequest called [" << name << "] at line " << __LINE__ << " of " << __FILE__ << " as the supplied function pointer was nil." << std::endl;
      success = false;
    };
    
    return success;
  };

  // The name that is supplied to this method should be the "bare" name of the test, i.e. leave out
  // any trailing "Test".  Example:  "StrobeDelay" not "StrobeDelayTest"
  Sct_CalibrationController::TestRequest_ptr TestLibraryImpl::getTestRequestByName(const char * name) {
    const std::string n = name; 
    const StringFunctionPtrMap::const_iterator it = stringFunctionPtrMap().find(n);

    Sct_CalibrationController::TestRequest_ptr ans = Sct_CalibrationController::TestRequest::_nil();

    if (it == stringFunctionPtrMap().end()) {
      std::ostringstream os;
      os << "Was asked in TestLibraryImpl::getTestRequestByName(const char * name) at line " << __LINE__ << " of " << __FILE__ << " for a TestRequest named [" << n << "] that doesn't exist, or rather has not yet been put into the map that records which ones exist!";
      Sct::SctNames::Mrs() << MRS_WARNING << "TEST_LIBRARY_IMPL" << MRS_TEXT(os.str()) << ENDM;
    } else {
      // Call the static instance method of the TestRequest to get hold of its instance.
      ans = ((it->second)());
      if (CORBA::is_nil(ans)) {
        // oh dear .. this should not happen .. something has gone wrong.
        std::ostringstream os;
        os << "I was asked in TestLibraryImpl::getTestRequestByName(const char * name) at line " << __LINE__ << " of " << __FILE__ << " for a TestRequest named [" << n << "].  This test was indeed registered with me, but the instance associated with the function pointer was nil.  There is something rotten in the state of Denmark.";
        Sct::SctNames::Mrs() << MRS_WARNING << "TEST_LIBRARY_IMPL" << MRS_TEXT(os.str()) << ENDM;
      };
    };

    return ans;
  }

  Sct_CalibrationController::TestLibrary::Strings* TestLibraryImpl::getTestNames() {
    const unsigned int siz=stringFunctionPtrMap().size();
    Sct_CalibrationController::TestLibrary::Strings*seq = Sct::SequenceUtils::newSafeSequence<Sct_CalibrationController::TestLibrary::Strings>(siz);
    unsigned int i=0;
    for (StringFunctionPtrMap::const_iterator it=stringFunctionPtrMap().begin();
         it!=stringFunctionPtrMap().end();
         (++it, ++i)) {
      (*seq)[i] = Sct::copyStringToCorba(it->first);
    };
    return seq;
  };
  
TestLibraryImpl::TestLibraryImpl() {
    std::cout << "create TestLibraryImpl" << std::endl;
}

Sct_CalibrationController::TestLibrary_ptr TestLibraryImpl::instance() {
    static TestLibraryImpl* instance = new TestLibraryImpl();
    return instance->_this();
}

}
