#ifndef SCTCALIBRATIONCONTROLLER_SCANREQUEST_H
#define SCTCALIBRATIONCONTROLLER_SCANREQUEST_H

#include "CalibrationController.hh"
#include <ipc/object.h>
#include <memory>
#include <string>
#include <vector>
#include "boost/shared_ptr.hpp"

using std::vector;
using std::auto_ptr;
using std::string;

namespace SctCalibrationController {
namespace Ipc {
    
class CalibrationController;
    
/**
  See the documentation in CalibrationController.idl for these methods.
  All may throw RequestException.  This is the nice interface
  */

class ScanRequest {
public:	    
    virtual CORBA::Boolean isRawNice()=0;
    virtual CORBA::Boolean configureModulesNice()=0;
    virtual long delayNice()=0;
    virtual void setScanNice(Sct_SctApi::Scan_ptr s)=0;
    virtual CORBA::Boolean clockByTwoNice()=0;
    virtual void setClockByTwoNice(CORBA::Boolean value)=0;
    virtual long widthNice()=0;
    virtual void setWidthNice(long value)=0;
    virtual Sct_SctApi::Scan_ptr getScanNice()=0;
};
 
 
 class MinimalScanRequest : public virtual ScanRequest {
 protected:
   MinimalScanRequest();
   virtual ~MinimalScanRequest() {};
 public:
   virtual CORBA::Boolean isRawNice();
   virtual CORBA::Boolean configureModulesNice();
   virtual CORBA::Boolean clockByTwoNice();
   virtual void setClockByTwoNice(CORBA::Boolean value);
   virtual CORBA::Long widthNice();
   virtual void setWidthNice(CORBA::Long width);
   virtual CORBA::Long delayNice();
   virtual void setScanNice(Sct_SctApi::Scan_ptr s);
   virtual Sct_SctApi::Scan_ptr getScanNice();
 protected:
   CORBA::Boolean m_clockByTwo;
   CORBA::Long m_width;
   Sct_SctApi::Scan_var m_scan;  
 };


/**
  This is the internal implementation, to be derived from;
  */
class ScanRequestImpl : 
  public IPCObject<POA_Sct_CalibrationController::ScanRequest>,
  protected MinimalScanRequest {
public:
     ScanRequestImpl();

    // Corba interface implementation ......

    virtual CORBA::Boolean isRawCorba(/*CalibrationControllerStatus *_status*/);
    virtual CORBA::Boolean configureModulesCorba(/*CalibrationControllerStatus *_status*/);
    virtual CORBA::Boolean clockByTwoCorba(/*CalibrationControllerStatus *_status*/);
    virtual CORBA::Long delayCorba(/*CalibrationControllerStatus *_status*/);
    virtual Sct_SctApi::Scan_ptr getScanCorba (/*CalibrationControllerStatus *_status*/);
    virtual void setScanCorba (/*CalibrationControllerStatus *_status, */Sct_SctApi::Scan_ptr scan);
    virtual void setClockByTwoCorba (/*CalibrationControllerStatus *_status, */CORBA::Boolean value);
    virtual CORBA::Long widthCorba(/*CalibrationControllerStatus *_status*/);
    virtual void setWidthCorba(/*CalibrationControllerStatus *_status, */CORBA::Long width);

    // "Pretty" implementation .........

  private:
    //const boost::shared_ptr<MinimalScanRequest> m_niceScanRequest;

    /*
    virtual CORBA::Boolean isRaw();
    virtual CORBA::Boolean configureModules();
    virtual CORBA::Boolean clockByTwo();
    virtual void setClockByTwo(CORBA::Boolean value);
    virtual long width();
    virtual void setWidth(long width);
    virtual long delay();
    virtual void setScan(Sct_SctApi_T_Scan* s);
    virtual Sct_SctApi_T_Scan* getScan ();
    */
 protected:
    /*
    bool m_clockByTwo;
    long m_width;
    Sct_SctApi_T_Scan* m_scan;  
    */
};

/**
  This is the adapter for external implementations.
  These calls may fail and throw RequestExceptions
  */

/*
class ScanRequestAdapter : public virtual ScanRequest {
 public:
  virtual ~ScanRequestAdapter() {};
public:
    ScanRequestAdapter(Sct_CalibrationController::ScanRequest_ptr req);
    virtual bool isRaw();
    virtual bool configureModules();
    virtual bool clockByTwo();
    virtual long delay();
    virtual void setScan(Sct_SctApi::Scan_ptr s);
    virtual Sct_SctApi::Scan_ptr getScan();
    virtual void setClockByTwo(CORBA::Boolean value);
    virtual long width();
    virtual void setWidth(long width);
private:
    Sct_CalibrationController::ScanRequest_var request;
};
*/

}
}
#endif //SCTCALIBRATIONCONTROLLER_SCANREQUEST_H
