#ifndef SCTCALIBRATIONCONTROLLER_TXCURRENTTEST_H
#define SCTCALIBRATIONCONTROLLER_TXCURRENTTEST_H

#include "../ipc/TestRequest.h"
#include "../ipc/CalibrationController.h"
#include "../ipc/ScanLibrary.h"
#include "../ipc/CalibrationController.hh"

#include "../SctApiAccessException.h"
#include "../SctApiCall.h"
#include "../CalibrationControllerImpl.h"
#include "Sct/LogicErrors.h"
#include "SctApi/extraScans.h"

#include <CommonWithDsp/ABCD/ABCDscans.h>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace std;
using namespace SctCalibrationController::Ipc;
using namespace SctCalibrationController;

namespace SctCalibrationController {
namespace Scripts {
    
class TxCurrentTest : public TestRequestImpl {
private:
   static const bool registeredInTestLibrary;

public:
   static Sct_CalibrationController::TestRequest_ptr instance() {
	static TxCurrentTest rt;
	return rt._this();
    }
   
   virtual void startTestRealImpl(Sct_SctApi::SctApiIPC_ptr the_api, string& name, unsigned short& testVariable, unsigned long& nScans, vector<double>& testPoints) {	
     this->api = Sct_SctApi::SctApiIPC::_duplicate(the_api);
     // No masked channels
     name = "TxCurrentTest";
     testVariable = ST_ELAPSED;
     nScans = 1;
     testPoints.push_back(nTriggers);
   }
   
   virtual Sct_CalibrationController::ScanRequest_ptr getNextScanRealImpl(unsigned long index) {
     Sct_CalibrationController::ScanLibrary_var sl = CalibrationControllerImpl::instance()->getScanLibrary();
     const bool issueSoftReset = true;
     Sct_CalibrationController::ScanRequest_ptr request = sl->rawScan(nTriggers, issueSoftReset);
     Sct_SctApi::Scan_var scan = request->getScanCorba();
       // Number of bytes to read from the conf register: There should be about 210 ish bytes.  We will read a few more just so we have some elbow room.  Rather arbitrarily, then we chose to read 256 bits:
     const int bitsToRead = 256;
     request->setWidthCorba(bitsToRead);
     
     // This next two executable lines (which disables data
     // mode, and thus enables SendConfRegister mode) 
     // is the second line which (beyond the test names!) 
     // should be the source of difference between THIS
     // test and the usual TxCurrentTest:
     APICALL(scan, setOption(Sct_SctApi::Scan::enableDataMode, 0), "Failed to disable dataMode (failed to enable SendConfRegIDMode)")
       
     APICALL(scan, configure(ST_TX_CURRENT, 0, 255, 5), "Failed to set Scan information")
     return request;
   }
    
    virtual bool canFeedbackRealImpl() {
	return true;
    }
    
    virtual void endTestRealImpl() {
      api = Sct_SctApi::SctApiIPC::_nil();
    }

private:
    TxCurrentTest() {}
    Sct_SctApi::SctApiIPC_var api;    
    static const int nTriggers = 10;
};
}
}
#endif //SCTCALIBRATIONCONTROLLER_TXCURRENTTEST_H
