
#include "SubscriptionManagerImpl.h"

#include "SpecialSubscribeRequest.h"
#include "Sct/MultiMessageDebugStream.h"
#include <signal.h>

void Sct::SubscriptionManagerImpl::subscribe_special(const char* serverPattern,
                                                     const char* objectPattern,
                                                     CORBA::Boolean server_is_regular_expression,
                                                     CORBA::Boolean object_is_regular_expression,
                                                     is::callback_ptr cb) {

  
  const NameOrRegexp srv(serverPattern, server_is_regular_expression);
  const NameOrRegexp obj(objectPattern, object_is_regular_expression);
  
  SpecialSubscribeRequest ssr(srv, obj, cb);

  Sct::MultiMessageDebugStream m;
  m << "SubscriptionManager recieved request to subscribe to [" << ssr << "] ... processing request ... ";
  m.flush();

  m_specialSubscribeRequests.push_back(ssr);

  m << "SubscriptionManager finished processing subscription to [" << ssr << "].";
  m.flush();

};

void Sct::SubscriptionManagerImpl::debugDumpSubscriptions() {
  Sct::MultiMessageDebugStream m;
  m << "SubscriptionManagerImpl has " << m_specialSubscribeRequests.size() << " specialSubscribeRequests registered at the moment. They are:";
  m.flush();

  {
    unsigned int i=0;
    for (SpecialSubscribeRequests::const_iterator it = m_specialSubscribeRequests.begin();
         it != m_specialSubscribeRequests.end();
         (++it, ++i)) {
      (m << i <<"\t" << (*it)).flush();
    };
  };
};

void Sct::SubscriptionManagerImpl::startCatchingSignals() {
  signal(SIGINT,  Sct::SubscriptionManagerImpl::acceptSignal);
  signal(SIGTERM, Sct::SubscriptionManagerImpl::acceptSignal);
};

void Sct::SubscriptionManagerImpl::acceptSignal(int signum) {
  std::cout << "Signal " << signum << " received at " << __FILE__ << " line " << __LINE__ << std::endl;
  if (signum==SIGINT || signum==SIGTERM) {
    std::cout << "Withdrawing from service due to signal " << signum << "." << std::endl;
  };
};

void Sct::SubscriptionManagerImpl::tidyUpAndWithdrawEtc() {
  this->withdraw();
};
