
#ifndef SCT_SUBSCRIPTIONMANAGER_H
#define SCT_SUBSCRIPTIONMANAGER_H

#include "SubscriptionManagerImpl.fwd"

#include <ipc/object.h>
#include "SubscriptionManager.hh"
#include "SpecialSubscribeRequest.h"

namespace Sct {
  
  class SubscriptionManagerImpl : public IPCNamedObject<POA_SubscriptionManagerModule::SubscriptionManager> {
  private:
    SubscriptionManagerImpl() : 
      IPCNamedObject<POA_SubscriptionManagerModule::SubscriptionManager>("SCT",
                                                                         SubscriptionManagerModule::SubscriptionManager::instanceName) {
    };
  public:
    static SubscriptionManagerImpl & instance() {
      static SubscriptionManagerImpl theInstance;
      return theInstance;
    };


    /////////////////////////////////////////////////////////////////////////////
    // Implementation of the methods declared in the SubscriptionManager.idl
    /////////////////////////////////////////////////////////////////////////////

    virtual CORBA::Long whatNumberIsThis(CORBA::Long number) {
      return number;
    };
    virtual void subscribe_special(const char* serverPattern,
                                   const char* objectPattern,
                                   CORBA::Boolean server_is_regular_expression,
                                   CORBA::Boolean object_is_regular_expression,
                                   is::callback_ptr cb);
    /// For debug purposes, debugDumpSubscriptions() should print to the implementations stdout information summarising the subscriptions that it is holding.
    virtual void debugDumpSubscriptions();

    /////////////////////////////////////////////////////////////////////////////
    // Other things
    /////////////////////////////////////////////////////////////////////////////

    virtual void shutdown() {
      // can't think of anything to do here, but this is a pure virtual method of IPCServant that we must implement.
    };
    /// We need to catch things like ctrl-c otherwise we won't be able to "withdraw" ourselves!
    void startCatchingSignals();
    ~SubscriptionManagerImpl() {
      std::cout << "Withdrawing from service due to demise of self." << std::endl;
      this->tidyUpAndWithdrawEtc();
    };

    static void acceptSignal(int signum);
 private:
    void tidyUpAndWithdrawEtc();
    typedef std::list<SpecialSubscribeRequest> SpecialSubscribeRequests;
    SpecialSubscribeRequests m_specialSubscribeRequests;
  };

};

#endif
