#include "is/isinfotrue.h"
#include "is/isdictionary.h"
#include "owl/time.h"

static char * my_strdup( const char * s )
{
    int i = strlen( s );
	
    char * d = new char[i+1];
	
    memcpy( d, s, i+1 );
    return d;
}

//////////////////////////////////////////////////
// ISDictionary methods implementation
//////////////////////////////////////////////////


void    ISDictionary::find ( const char * name, CORBA::String_out data )
{    
    std::string key = name;
    
    OWLMutexLock ml( mutex_ );
    
    InfoMap::iterator it = information_.find( key );
    ISInfoTrue * info = ( it != information_.end() ? (*it).second : 0 );
    
    if ( info != 0 )
    {
	data = my_strdup( info -> data() );
    }
    else
    {
	if ( verbosity_ > 0 )
	{
	    std::cout << OWLTime() << " :: WARNING [ISDictionary::find] Information object " << name << " does not exist" << std::endl;
	}
	throw is::NotFound();
    }
}

void  ISDictionary::destroy( )
{
    if ( --ref_cnt_ == 0 )
    {
	OWLMutexLock ml( mutex_ );
	InfoMap::iterator it = information_.begin( );
	while ( it != information_.end( ) )
	{
	    ISInfoTrue * info = (*it).second;
	    info -> freeze( false );
	    if (    info -> state() == ISInfoTrue::Private 
		 && !info -> hasSubscriptions() 
		 && !info -> frozen() )
	    {
		information_.erase( it++ );
		delete info;
	    }
	    else
		++it;
	}
    }
}
