#include "OmniMarshalling_Test.h"
#include "../src/OmniMarshalling.h"
#include "marshalling.hh"
#include <boost/test/unit_test.hpp>
#include <boost/thread.hpp>
#include "ipc/object.h"
#include "ipc/server.h"

class MyImpl : public IPCNamedObject<POA_marshalling::SequenceInterface, ipc::single_thread>{
private:
  struct Data{
    Data(long l, std::string s) : l(l), s(s) {}
    long l;
    std::string s;
  };
  std::list<Data*> datalist;
  std::list<std::string> stringList;
public:
  MyImpl(IPCPartition& p, const char* name) : 
    IPCNamedObject<POA_marshalling::SequenceInterface, ipc::single_thread>(p,name)
  {
    
    std::cout << "made a SequenceInterface" << std::endl;
  }
  
  void shutdown() {}
  
  virtual ~MyImpl(){
    for (std::list<Data*>::iterator i=datalist.begin();
         i!=datalist.end();
         ++i){
      delete *i;
    }
  }
  void add(CORBA::Long mylong, const char* myString){
    datalist.push_back(new Data(mylong, myString));
    stringList.push_back(myString);
  }

  void getFirst(CORBA::Long& mylong, CORBA::String_out mystring){
    if (datalist.empty()) return;
    mylong = (*datalist.begin())->l;
    mystring = CORBA::string_dup( (*datalist.begin())->s.c_str());
  }

  virtual char* echoString(const char* echo){
    return Sct::copyStringToCorba(echo);
  }

  virtual char* getName(){
    return Sct::copyStringToCorba("Charlie");
  }
  
  marshalling::StringList* returnStringList(){
    return Sct::copyStringListToCorba<marshalling::StringList>(stringList);
  }

  marshalling::LongList* returnLongList(){
    marshalling::LongList* newlist = new marshalling::LongList(datalist.size());
    newlist->length(datalist.size());
    unsigned i=0;
    for (std::list<Data*>::const_iterator it=datalist.begin();
         it != datalist.end();
         ++it){
      (*newlist)[i]=(*it)->l;
       ++i;
    }
    return newlist;
  }

  marshalling::StructList* returnStructList(){
    std::cout << "in returnStructList()" << std::endl;
    marshalling::StructList* newlist = new marshalling::StructList(datalist.size());
    newlist->length (datalist.size());
    std::cout << "made new StructList" << std::endl;
    unsigned i=0;
    for (std::list<Data*>::const_iterator it=datalist.begin();
         it != datalist.end();
         ++it){
      (*newlist)[i].aLong=(*it)->l;
      (*newlist)[i].aString=CORBA::string_dup((*it)->s.c_str());
      ++i;
    }
    return newlist;
  }
  
};

void testStringReturn(){
  std::string echo_me("blah");
  std::cout << "in function handle = " << Sct::OmniMarshalling_Test::handle << std::endl;
  try{
    CORBA::String_var echo = Sct::OmniMarshalling_Test::handle->echoString(echo_me.c_str());
    BOOST_CHECK_EQUAL(std::string(echo),echo_me);
  }catch(CORBA::SystemException& e){
    std::cerr << "testStringReturn CORBA::SystemException " << e._name() << std::endl;
    throw;
  }catch(CORBA::UserException& e){
    std::cerr << "testStringReturn CORBA::UserException " <<  e._name() << std::endl;
    throw;
  }
}

void testName(){
   try{
     CORBA::String_var name = Sct::OmniMarshalling_Test::handle->getName();
     BOOST_CHECK_EQUAL(std::string("Charlie"),std::string(name));
   }catch(CORBA::SystemException& e){
     std::cerr << "testStringReturn CORBA::SystemException " << e._name() << std::endl;
     throw;
   }catch(CORBA::UserException& e){
     std::cerr << "testStringReturn CORBA::UserException " <<  e._name() << std::endl;
     throw;
   }
}

marshalling::SequenceInterface_ptr Sct::OmniMarshalling_Test::handle;

void functionTest(marshalling::SequenceInterface_ptr handle){
  try{
    CORBA::String_var echo = handle->echoString("success") ;
    std::cout << "functionTest echos " << echo << std::endl;
  }catch(CORBA::Exception &e){
    std::cerr << "functionTest throws Corba::Exception " << e._name() << std::endl;
    throw;
  }
}

static marshalling::SequenceInterface_ptr static_handle;

void staticTest(){
  try{
    CORBA::String_var echo = static_handle->echoString("success") ;
    std::cout << "staticTest echos " << echo << std::endl;
  }catch(CORBA::Exception &e){
    std::cerr << "staticTest throws Corba::Exception " << e._name() << std::endl;
    throw;
  }
}

void testListOfString(){
   try{
       std::cout << "trying returnStringList " << static_handle << std::endl;
       BOOST_CHECK(static_handle);
       marshalling::StringList* list = static_handle->returnStringList();
       BOOST_CHECK_EQUAL(list->length(),3);
       for (unsigned i=0; i<list->length(); ++i){
	       std::string cont((*list)[i]);
      	       BOOST_CHECK(cont.find("blah")!=std::string::npos);
       } 
       delete list;
   }catch(CORBA::Exception &e){
       std::cerr << "testListOfString throws Corba::Exception " << e._name() << std::endl;
       throw;
   }

}


namespace Sct {

  OmniMarshalling_Test::OmniMarshalling_Test() {
    IPCPartition p("BLAH");
    impl = new MyImpl(p, "MyImpl");
    handle = impl->_this();
    CORBA::String_var echo = handle->echoString("success") ;
    std::cout << "direct use of handle: " << echo << std::endl;
    std::cout << "handle = " << handle << std::endl;
    std::string blah="blah de blah";
    functionTest(handle);
    static_handle=handle;
	    
    add(BOOST_TEST_CASE(&testStringReturn));
    
    handle->add(1,"blah");
    handle->add(2,"second blah");
    handle->add(3,"yet more blah");
    add(BOOST_TEST_CASE(&testListOfString));
    add(BOOST_TEST_CASE(&testName));
    // One must not put a line like the following:
    //    impl->_destroy();
    // in this part of the code, as the "add(BOOST_TEST_CASE(...))" lines
    // above have only SCHEDULED the running of some tests.  The Boost
    // mail program will only RUN those tests once this constructor has completed, and even then, it has (in principle) the freedom to wait 10 mins if it so desires before actually performing them.  We may only _destroy() the impl in the destructor!
  } 
  
  OmniMarshalling_Test::~OmniMarshalling_Test() {
    impl->_destroy();
  }
}

