#include "../OpeResult.h"
#include "../RootStreamableAdapter.h"
#include "OpeStreamer_v1.h"
#include "../OpeResultIOHelper.h"
#include "Sct/SctParameters.h"
#include "Sct/LogicErrors.h"

#include <TGraph.h>
#include <TH1.h>

using namespace Sct;

namespace SctData {
namespace IO {

/* READ NOTES ON STREAMERS AND VERSIONS BEFORE EDITING THIS FILE! */
unsigned OpeStreamer_v1::s_version=1;
	
OpeStreamer_v1::OpeStreamer_v1() throw() {}

bool OpeStreamer_v1::inMap = IOManager::addToMap("SctData::OpeResult",  auto_ptr<Streamer>(new OpeStreamer_v1()));

shared_ptr<Streamable> OpeStreamer_v1::read(IStream& in, const IOManager& manager) const throw(LogicError, IoError) {
    OpeResultIOHelper helper;
    auto_ptr<SctData::OpeResult> ope = helper.create();
    shared_ptr<Sct::Streamable> ad(ope);
    read(in, *ad, manager);
    return ad;
}

void OpeStreamer_v1::write(OStream& out, const Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    const OpeResult& myope = dynamic_cast<const OpeResult&>(ob);
    out << myope.getUniqueID();
    for (unsigned int ichip=0; ichip<nChipModule; ichip++) {
        writeChipResult(out, myope.getChipResult(ichip), manager);
    }
}

void OpeStreamer_v1::read(IStream& in, Streamable& ob, const IOManager& manager) const throw(LogicError, IoError) {
    OpeResult& myope = dynamic_cast<OpeResult&>(ob);
    std::string uniqueid;
    in >> uniqueid;
    OpeResultIOHelper help;
    help.set(myope);
    help.setUniqueID(UniqueID(uniqueid));
    for (unsigned int ichip=0; ichip<nChipModule; ichip++) {
        readChipResult(in, myope.getChipResult(ichip), manager);
    }
}

void OpeStreamer_v1::writeChipResult(OStream& out, const ChipOpeResult& c, const IOManager& manager) const throw (LogicError, IoError) {
    out << (bool) c.hist.get();
    if (c.hist.get()){
      manager.writeImpl(out, RootStreamableAdapter( *c.hist ), true);
      out << c.badmax << c.threshold;
    }
}

void OpeStreamer_v1::readChipResult(IStream& in, ChipOpeResult& c, const IOManager& manager) const throw(LogicError, IoError) {
    bool available;
    in >> available;
    if (available){
      shared_ptr<Streamable> ob = manager.readImpl(in);
      shared_ptr<RootStreamableAdapter> ad  = boost::dynamic_pointer_cast<RootStreamableAdapter>(ob);
      c.hist = shared_ptr<TH1> ( dynamic_cast<TH1*>(ad->getObject()));
      in >> c.badmax >> c.threshold;
    }
}

}
}
