#ifndef SCTTEST_HIGHLEVELAPI_H
#define SCTTEST_HIGHLEVELAPI_H

#include <string>
#include <memory>
#include <ipc/object.h>
#include <boost/shared_ptr.hpp>
#include "ModuleData.h"
#include "SctTestApi.hh"

using std::string;

namespace SctData {
    class ModuleConfiguration;
    class ConfigurationVariable;
}
class TestData;

// Obsolete? extern IPCServer ipcServer;

namespace SctTest {

class HighLevelApi : public IPCNamedObject<POA_SctTestApi::HighLevelApi> {
public:
    static HighLevelApi& instance();

    // "Handy" methods:
    //
    /**
    Sets the serial number and loads in all necessary data.
    @throws NoDataException if it can't find any valid data.
    */
    void handy_setModuleSerial(const string & serial);
    string handy_getModuleSerial() const;

    /**
     * Sets the data file.
     * @throws Sct::FileException if there is a problem with the file
     */
    void handy_setDataFile(const string & file);
    string handy_getDataFile() const;

    /**
     * Sets the XML summary file
     * @throws Sct::FileException if there is a problem with the file
     */
    void handy_setXmlSummaryFile(const string & file);
    string handy_getXmlSummaryFile() const;

    //CORBA methods
    virtual void setDataFile (const char * file);
    virtual char * getDataFile ();
    virtual void setXmlSummaryFile (const char * file);
    virtual char * getXmlSummaryFile ();
    virtual void setModuleSerial (const char * serial);
    virtual char * getModuleSerial ();
    virtual void fullBypass();
    virtual void timeWalk();
    virtual void strobeDelay();
    virtual void threePointGain();
    virtual void responseCurve();
    virtual void noiseOccupancy();
    virtual void trim();
    virtual void pipeline();
    virtual void shutdown() {};

private:
    HighLevelApi();

    /**
    Gets the ModuleConfiguration for the given serial number.
    @throws NoDataException if it can't get any data
    */
    void getConfig(string serial);
    void addAndGetConfig(string serial);
    void publishTest(TestData& t) const;
    string getFileName(const TestInfo& info, unsigned int scanNumber) const;
    void doTest(const TestInfo& info, TestData& data, const SctData::ConfigurationVariable& var) const;
    void publishScan(const TestInfo& info, unsigned int scanNumber, const SctData::ConfigurationVariable& scan_variable, const SctData::ModuleConfiguration& config) const;

    string serial;
    string xmlFile;            ///<The xml summary file
    ModuleData data;
    boost::shared_ptr<SctData::ModuleConfiguration> config;
};
}

#endif //SCTTEST_HIGHLEVELAPI_H
