#include "TrimRangeDisplayer.h"
#include "DisplayManager.h"
#include "DisplayInfo.h"

#include "SctData/TrimRangeTestResult.h"
#include "SctData/FitScanResult.h"
#include "Sct/SctParameters.h"

#include <TGraph.h>
#include <TCanvas.h>
#include <TH2.h>
#include <TF1.h>
#include <TLatex.h>
#include <TLine.h>

#include <iomanip>
#include <algorithm>

using namespace Sct;
using namespace SctData;
using namespace std;
using namespace boost;

namespace SctDataDisplay {
    
class TrimDisplayData : public DisplayData {
public:
    vector<shared_ptr<TCanvas> > canvas;
};   
    
bool TrimRangeDisplayer::inMap = DisplayManager::addToMap("SctData::TrimRangeTestResult", shared_ptr<Displayer>(new TrimRangeDisplayer()));    
    
shared_ptr<DisplayData> TrimRangeDisplayer::display(shared_ptr<const Sct::Serializable> serial, const DisplayInfo& info, std::ostream& os) {    
    shared_ptr<TrimDisplayData> data (new TrimDisplayData());
    shared_ptr<const TrimRangeTestResult> result = dynamic_pointer_cast<const TrimRangeTestResult>(serial);

    shared_ptr<TCanvas> c = createCanvas((string)result->getHeader().getUniqueID() + "_TrimData", result->getModuleName() + " Trim Range Fits");
    data->canvas.push_back(c);
    c->Divide(4, 3);
    for (unsigned int ichip = 0; ichip < nChipModule; ++ichip) {
        c->cd(ichip + 1);
	gPad->SetLeftMargin(0.17);
	gPad->SetBottomMargin(0.15);
	gPad->SetTopMargin(0.03);
	gPad->SetRightMargin(0.03);

	// Find max for plot:
	float histmax=100.;
	float histmin=1.0E7;
	for (unsigned ichannel=0; ichannel<nChannelChip; ++ichannel){
	  Stat<TrimRangeTestResult::TrimData> trim = result->chipTrimData[ichip]->channelData.getAt(ichannel);
	  for (unsigned ipt=0; ipt<trim.value.graph.size(); ++ipt){
	    if (trim.value.graph[ipt].first > histmax ) {
	      histmax=trim.value.graph[ipt].first;
	    }
	    if (trim.value.graph[ipt].first < histmin ) {
	      histmin=trim.value.graph[ipt].first;
	    }
	  }
	}
	
	char name[100]; sprintf(name,"Chip %d",ichip);
	TH2D* hist = new TH2D(name,name,10,histmin*0.9,histmax*1.1,10,0,16);
	hist->GetYaxis()->SetTitleOffset(1.2);
	hist->GetXaxis()->SetLabelSize(0.06);
	hist->GetYaxis()->SetLabelSize(0.06);
	hist->GetXaxis()->SetTitleSize(0.06);
	hist->GetYaxis()->SetTitleSize(0.06);
	hist->GetXaxis()->CenterTitle();
	hist->GetYaxis()->CenterTitle();
	hist->SetXTitle("VT50");
	hist->SetYTitle("Trim DAC");
	hist->Draw();

	int range=result->chipTrim[ichip]->range;
	char txt[10];
	sprintf(txt,"R%d",range);
	TLatex* lab=new TLatex();
	lab->SetTextSize(0.1);
	lab->DrawLatex(0,0,txt);

	for (unsigned ichannel=0; ichannel<nChannelChip; ++ichannel){
	    if (result->chipTrimData[ichip]!=0){
		Stat<TrimRangeTestResult::TrimData> trim = result->chipTrimData[ichip]->channelData.getAt(ichannel);
		TGraph* g = new TGraph();
		g->SetMarkerColor(ichannel);
		g->SetMarkerStyle(20);
		g->SetMarkerSize(0.5);
		g->Set(trim.value.graph.size());
		for (unsigned ipt=0; ipt<trim.value.graph.size(); ++ipt){
		    g->SetPoint(ipt,trim.value.graph[ipt].first, trim.value.graph[ipt].second);
		}
		g->SetLineColor(ichannel);
		g->SetLineWidth(1);
		if (g->GetN()!=0 ){
		    g->Draw("LP");
		    TH1* hist=g->GetHistogram();
		    hist->GetYaxis()->SetTitleOffset(1.2);
		    hist->GetXaxis()->SetLabelSize(0.06);
		    hist->GetYaxis()->SetLabelSize(0.06);
		    hist->GetXaxis()->SetTitleSize(0.06);
		    hist->GetYaxis()->SetTitleSize(0.06);
		    hist->GetXaxis()->CenterTitle();
		    hist->GetYaxis()->CenterTitle();
		}
	    }
	}
    }
  
    shared_ptr<TCanvas> c2 = createCanvas((string)result->getHeader().getUniqueID() + "_Trims", result->getModuleName() + " Trims");
    data->canvas.push_back(c2);
    c2->Divide(1, 4);
    for (unsigned int ilink = 0; ilink < nLinkModule; ++ilink) {
	TGraph* g1 = new TGraph();
	TGraph* g2 = new TGraph();
	g1->SetMarkerColor(2);
	g1->SetMarkerStyle(20);
	g1->SetMarkerSize(0.7);
	g2->SetMarkerColor(4);
	g2->SetMarkerStyle(20);
	g2->SetMarkerSize(0.7);

	float maxvt=0;
	float minvt=1.0E10;

	// find number of trimmed channels:
	int npoints=0;
	for (unsigned ichip=ilink*nChipLink; ichip<(ilink+1)*nChipLink; ++ichip){
	    if (result->chipTrim[ichip]==0){
		cerr << " no trim at chip "<< ichip << endl; continue;
	    }
	    npoints += result->chipTrim[ichip]->channelTrim.n();
	}
	
	g1->Set(npoints);
	g2->Set(npoints);

	int ipoint=0;
	for (unsigned ichannel=ilink*nChannelLink; ichannel<(ilink+1)*nChannelLink; ++ichannel){

	    if (result->chipTrim[ichannel/128]==0){
		continue;
	    }

	    Stat<TrimRangeTestResult::Trim> trim = result->chipTrim[ichannel/128]
		->channelTrim.getAt(ichannel%128);

	    if (trim.valid){
		g1->SetPoint(ipoint,ichannel%nChannelLink,trim.value.trim);
		g2->SetPoint(ipoint,ichannel%nChannelLink,trim.value.vthr);
		++ipoint;
		if (trim.value.vthr > maxvt){
		  maxvt=trim.value.vthr;
		}
		if (trim.value.vthr < minvt){
		  minvt=trim.value.vthr;
		}
	    }
	}
	
	char name[100]; sprintf(name,"Link %d TrimDAC",ilink);
	TH2D* hist1 = new TH2D(name,name,768,-0.5,767.5,16,-0.5,15.5);
	sprintf(name,"Link %d VT50",ilink);
	TH2D* hist2 = new TH2D(name,name,768,-0.5,767.5,3,minvt*0.9,maxvt*1.1);
	c2->cd(ilink*2 + 1);
	configureTrimHist(*hist1);
	hist1->SetYTitle("Trim DAC");
	hist1->Draw();
	g1->Draw("P");
	for (unsigned ichip=ilink*nChipLink; ichip<(ilink+1)*nChipLink; ++ichip){
	  TAxis* ax=hist1->GetYaxis();
	  if (ichip%nChipLink!=0){
	    TLine* l = new TLine();
	    l->SetLineStyle(2);
	    l->SetLineWidth(1);
	    float xline=(ichip%nChipLink)*nChannelChip;
	    std::cout << ichip << " xline=" << xline << std::endl;
	    l->DrawLine(xline,ax->GetXmin(),xline,ax->GetXmax());
	  }
	}

	c2->cd(ilink*2 + 2);
	configureTrimHist(*hist2);
	hist2->SetYTitle("VT50 after trim");
	hist2->Draw("");
	g2->Draw("P");

	// Draw on chip boundries and range labels.
	for (unsigned ichip=ilink*nChipLink; ichip<(ilink+1)*nChipLink; ++ichip){
	    int range=result->chipTrim[ichip]->range;
	    char txt[10];
	    sprintf(txt,"Range %d",range);
	    TLatex* lab=new TLatex();
	    lab->SetTextSize(0.08);
	    TAxis* ax=hist2->GetYaxis();
	    float y=ax->GetXmin() + (ax->GetXmax() - ax->GetXmin())*0.02;
	    lab->DrawLatex(nChannelChip*(ichip%nChipLink)+nChannelChip/2,y,txt);
	    if (ichip%nChipLink!=0){
	      TLine* l = new TLine();
	      l->SetLineStyle(2);
	      l->SetLineWidth(1);
	      float xline=(ichip%nChipLink)*nChannelChip;
	      l->DrawLine(xline,ax->GetXmin(),xline,ax->GetXmax());
	    }
	}

    }    
    
    //Now print out defects:
    os << endl << "Defects: " << endl;
    printDefectList(result->getDefects(), os);

    return data;
}

  void TrimRangeDisplayer::configureTrimHist(TH1& h){
    gPad->SetLeftMargin(0.05);
    gPad->SetBottomMargin(0.15);
    gPad->SetTopMargin(0.03);
    gPad->SetRightMargin(0.03);
    h.GetXaxis()->CenterTitle();
    h.GetYaxis()->CenterTitle();
    h.GetYaxis()->SetTitleOffset(0.3);
    h.GetXaxis()->SetLabelSize(0.08);
    h.GetYaxis()->SetLabelSize(0.08);
    h.GetXaxis()->SetTitleSize(0.08);
    h.GetYaxis()->SetTitleSize(0.08);
    h.SetXTitle("Channel");
  }

}
