/*
 * ConfigurationListener.java
 *
 * 
 */

package DisplayGUI;

import Sct.IS.SctNames;
import is.*;
import rc.RCStateInfo;
import dsa.DSAApplication;


public class ConfigurationListener implements InfoListener {
    
    gui guiControl;
    String cc_status="none";
    String rc_status="none";

    public ConfigurationListener(gui parent) {
        guiControl=parent;
        is = GuiComponents.System.ISInterface.getInstance();
        subscribe();
    }
    
    private void subscribe() {     


        try {
           is.subscribe("ConfigurationServer","ModuleConfChangeCount",false,this);
           is.subscribe("ConfigurationServer","ModuleMappingChangeCount",false,this);
           is.subscribe("ConfigurationServer","SystemStructureChangeCount",false,this);
        } catch (RuntimeException e2) {
            System.err.println("SctGUI::ConfigurationListener::Couldn't subscribe to ConfigurationServer: " + e2.toString());
        }
        try {
          is.subscribe("RunCtrl","Application_CalibrationController",false,this);
          is.subscribe("RunCtrl","SCTRootSegmentController",false,this);
        } catch (RuntimeException e3) {
            System.err.println("SctGUI::ConfigurationListener::Couldn't subscribe to RunCtrl Applications: " + e3.toString());
        }
    }
    
    void close() {


        try {
            is.unsubscribe("ConfigurationServer","ModuleConfChangeCount");
            is.unsubscribe("ConfigurationServer","ModuleMappingChangeCount");
            is.unsubscribe("ConfigurationServer","SystemStructureChangeCount");
            is.unsubscribe("RunCtrl","Application_CalibrationController");
	    is.unsubscribe("RunCtrl","SCTRootSegmentController");
        } catch (RuntimeException e2) {
            System.err.println("Couldn't unsubscribe from ConfigurationServer: " + e2.getMessage());
        }
    }

    abstract class StringRunnable implements Runnable {
	protected String name;
	StringRunnable(String name) {
	    this.name = name;
	}
    }

    public void infoCreated(is.InfoEvent infoEvent) {
//       javax.swing.SwingUtilities.invokeLater(new StringRunnable(infoEvent.getName()) {
//          public void run() {
//                  if(!name.equals("RunCtrl.RootCtrl")) resetDisplayConfiguration();
//             }
//        });
        infoUpdated(infoEvent);
    }

    public void infoDeleted(is.InfoEvent infoEvent) {
      infoUpdated(infoEvent);
    }

    public void infoUpdated(is.InfoEvent infoEvent) {
//       System.out.println("*** got "+infoEvent.getName());
       javax.swing.SwingUtilities.invokeLater(new StringRunnable(infoEvent.getName()) {
          public void run() {
             if(name.startsWith("ConfigurationServer")) resetDisplayConfiguration();
	     else if(name.matches(".*SCTRootSegmentController.*")) {
	        if(GuiComponents.System.ISInterface.getInstance().getRepository().contains(name)) {
                  RCStateInfo rc = new RCStateInfo();
                  GuiComponents.System.ISInterface.getInstance().getRepository().getValue(name, rc);
		  if(rc.state.matches("CONFIGURED") && rc_status.matches("INITIAL|LOADED")) resetDisplayConfiguration();
		  rc_status=rc.state;
		  }
                else rc_status="none";
//                System.out.println("RC moved to state "+rc_status);
		}
             else if(name.matches(".*Calibration.*")) {
                DSAApplication rc = new DSAApplication();
                GuiComponents.System.ISInterface.getInstance().getRepository().getValue(name, rc);
//		System.out.println("CC moved to state "+rc.status);
                if(rc.status.matches("IDLE|RUNNING") && !cc_status.equals(rc.status)) {
		      resetCCPanel();
		      cc_status=rc.status;
		      }
		}

             }
        });
    }


    private void resetCCPanel() {
       GuiComponents.System.SystemInterface.getInstance().refresh();
       GuiComponents.System.ISInterface.getInstance().refresh();
       guiControl.calibrationControllerPanel.subscribeToServers(); // fix for loss of contact
       }
       
    private void resetDisplayConfiguration() {
       GuiComponents.System.SystemInterface.getInstance().refresh();
       GuiComponents.System.ISInterface.getInstance().refresh();
       ConfigurationInterface.getInstance().Refresh();
       guiControl.displayStatus.setConfigurationMode();
       int nmods = ConfigurationInterface.getInstance().getTotalNumberOfModules();
       System.out.println("Configuration updated with "+nmods+" modules.");
       int cacheSize = nmods<200 ? 200 : nmods;
       guiControl.isInterface.setCacheSize(cacheSize);
       }


     private GuiComponents.System.ISInterface is;


 
}


